# coding: utf-8

import os
import cgitb
from threading import Thread
import time
import logging

import tornado.ioloop
import tornado.web
import tornado.httpserver

from agentshandler import AgentsHandler
from brousershandler import BrowsersHandler
from sharedvariables import agents_list
from sockjs.tornado import SockJSRouter
import settings

AGENTS_SOCKET_PORT = 8001
BROWSERS_SOCKET_PORT = 8002
STATIC_PATH = os.path.join(os.path.dirname(__file__), 'static')

class HttpServerThread(Thread):
    def __init__(self, urls, port, *args, **kwargs):
        try:
            Thread.__init__(self, *args, **kwargs)
            app = tornado.web.Application(urls)
            http_server = tornado.httpserver.HTTPServer(app)
            http_server.listen(port)
            self._stop = False
            self.daemon = True
        except Exception as e:
            logging.error(e)
            logging.error(cgitb.text(sys.exc_info()))

    def stop(self):
        self._stop = self.__stop()
        self._loop.stop()
    
    def run(self):
        try:
            while not self._stop:
                try:
                    self._loop = tornado.ioloop.IOLoop.instance()
                    self._loop.start()
                except KeyboardInterrupt:
                    break
        except Exception as e:
            logging.error(e)
            logging.error(cgitb.text(sys.exc_info()))
            
class SockjsServerThread(Thread):
    def __init__(self, handler, url, port, *args, **kwargs):
        try:
            Thread.__init__(self, *args, **kwargs)
            EchoRouter = SockJSRouter(handler, url)
            app = tornado.web.Application(EchoRouter.urls)
            app.listen(port)
            self._stop = False
            self.daemon = True
        except Exception as e:
            logging.error(e)
            logging.error(cgitb.text(sys.exc_info()))

    def stop(self):
        self._stop = self.__stop()
        self._loop.stop()
    
    def run(self):
        try:
            while not self._stop:
                try:
                    self._loop = tornado.ioloop.IOLoop.instance()
                    self._loop.start()
                except KeyboardInterrupt:
                    break
        except Exception as e:
            logging.error(e)
            logging.error(cgitb.text(sys.exc_info()))
    
def ensure_dir(d):
    if not os.path.exists(d):
        os.makedirs(d)
    
def start_server_agents():
    ensure_dir(settings.RECIPES_PATH)
    agents_urls = [(r'/agent-server', AgentsHandler), ]
    agents_server_thread = HttpServerThread(agents_urls, AGENTS_SOCKET_PORT)
    agents_server_thread.start()
    
def start_server_browsers():
    agents_server_thread = SockjsServerThread(BrowsersHandler, '/socket', BROWSERS_SOCKET_PORT)
    agents_server_thread.start()
    
def start_server():
    start_server_agents()
    start_server_browsers()
        
if __name__ == '__main__':
    start_server()
